import json
import os
from unittest import mock

import pytest

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.util.env_vars_config import env_vars_config
from checkov.common.util.json_utils import object_hook, CustomJSONEncoder
from checkov.terraform import TFModule
from checkov.terraform.graph_builder.foreach.abstract_handler import ForeachAbstractHandler
from checkov.terraform.graph_builder.foreach.builder import ForeachBuilder
from checkov.terraform.graph_builder.foreach.module_handler import ForeachModuleHandler
from checkov.terraform.graph_builder.foreach.resource_handler import ForeachResourceHandler
from checkov.terraform.graph_builder.graph_to_tf_definitions import convert_graph_vertices_to_tf_definitions

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


def load_expected_data(path):
    dir_name = os.path.realpath(os.path.join(TEST_DIRNAME, path))
    with open(dir_name, "r") as f:
        return json.load(f)


def assert_object_equal(res, expected_res):
    assert len(res) == len(expected_res)
    if isinstance(res, dict):
        assert dict(sorted(res.items(), key=lambda item: item[0])) == dict(sorted(expected_res.items(), key=lambda item: item[0]))
    if isinstance(res, list):
        assert res.sort() == expected_res.sort()


def build_and_get_graph_by_path(path, render_var=False):
    from checkov.terraform.graph_manager import TerraformGraphManager
    resources_dir = os.path.realpath(os.path.join(TEST_DIRNAME, 'resources', path))
    graph_manager = TerraformGraphManager('m', ['m'])
    local_graph, tf_definitions = graph_manager.build_graph_from_source_directory(resources_dir, render_variables=render_var)
    return local_graph, tf_definitions


@pytest.fixture()
def checkov_source_path() -> str:
    return os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(__file__)))))


@pytest.mark.parametrize(
    "block_index,expected_res,obj",
    [
        (0, ['bucket_a', 'bucket_b'], True),
        (1, {'key1': '${var.a}', 'key2': '${var.b}'}, True),
        (2, None, False),
        (3, None, False),
        (4, 5, False),
        (5, None, False),
        (6, None, False)
    ]
)
@mock.patch.dict(os.environ, {"CHECKOV_ENABLE_FOREACH_HANDLING": "False"})
def test_static_foreach_resource(block_index, expected_res, obj):
    dir_name = 'foreach_resources/static_foreach_value'
    local_graph = build_and_get_graph_by_path(dir_name)[0]
    foreach_handler = ForeachResourceHandler(local_graph)
    res = foreach_handler._get_static_foreach_statement(block_index)
    if obj:
        assert_object_equal(res, expected_res)
    else:
        assert res == expected_res


@mock.patch.dict(os.environ, {"CHECKOV_ENABLE_FOREACH_HANDLING": "False"})
def test_dynamic_foreach_resource():
    dir_name = 'foreach_resources/dynamic_foreach_value'
    local_graph = build_and_get_graph_by_path(dir_name)[0]
    foreach_handler = ForeachResourceHandler(local_graph)
    res = foreach_handler._handle_dynamic_statement([4, 5, 6, 7, 8])
    expected_res = {
        4: {'a_group': 'eastus', 'another_group': 'westus2'}, 5: ['s3-bucket-a', 's3-bucket-b'], 6: 5, 7: 2, 8: None
    }
    assert_object_equal(res, expected_res)


@mock.patch.dict(os.environ, {"CHECKOV_ENABLE_FOREACH_HANDLING": "False"})
def test_foreach_resource():
    dir_name = 'foreach_resources'
    local_graph = build_and_get_graph_by_path(dir_name)[0]
    foreach_handler = ForeachResourceHandler(local_graph)
    res = foreach_handler._get_statements([6, 7, 8, 9, 10, 19, 20, 21, 22, 23, 24, 25])
    expected_res = {
        4: {'a_group': 'eastus', 'another_group': 'westus2'},
        5: ['s3-bucket-a', 's3-bucket-b'],
        6: 5,
        7: 2,
        8: None,
        17: ['bucket_a', 'bucket_b'],
        18: {'key1': '${var.a}', 'key2': '${var.b}'},
        19: None,
        20: None,
        21: 5,
        22: None,
        23: None
    }
    for key, _ in expected_res.items():
        if isinstance(expected_res[key], (list, dict)):
            assert_object_equal(res[key], expected_res[key])
        else:
            assert res[key] == expected_res[key]


@mock.patch.dict(os.environ, {"CHECKOV_ENABLE_FOREACH_HANDLING": "False"})
def test_build_sub_graph():
    dir_name = 'foreach_resources'
    local_graph = build_and_get_graph_by_path(dir_name)[0]
    foreach_handler = ForeachAbstractHandler(local_graph)
    blocks = [6, 7, 8, 9, 10, 21, 22]
    sub_graph = foreach_handler._build_sub_graph(blocks)
    assert all(sub_graph.vertices[i] for i in blocks)
    assert not all(sub_graph.vertices[i] for i in range(len(sub_graph.vertices)))
    assert len(sub_graph.edges) < len(local_graph.edges)


def test_new_resources_count():
    dir_name = 'foreach_examples/count_dup_resources'
    local_graph = build_and_get_graph_by_path(dir_name)[0]
    main_count_resource = 'aws_s3_bucket.count_var_resource'

    foreach_builder = ForeachBuilder(local_graph)
    foreach_builder._module_handler.local_graph.enable_foreach_handling = True
    foreach_builder.handle({'resource': [3], 'module': []})
    for i, resource in enumerate([local_graph.vertices[1], local_graph.vertices[6], local_graph.vertices[7]]):
        assert resource.name.endswith(f"[{i}]")
        assert resource.id.endswith(f"[{i}]")
        assert list(resource.config['aws_s3_bucket'].keys())[0].endswith(f'[{i}]')
    new_vertices_names = [vertice.name for vertice in local_graph.vertices]
    assert main_count_resource not in new_vertices_names


def test_new_resources_foreach():
    dir_name = 'foreach_examples/foreach_dup_resources'
    local_graph = build_and_get_graph_by_path(dir_name)[0]
    foreach_builder = ForeachBuilder(local_graph)
    foreach_builder._module_handler.local_graph.enable_foreach_handling = True
    foreach_builder.handle({'resource': [0, 1], 'module': []})
    for resource in [local_graph.vertices[0], local_graph.vertices[1], local_graph.vertices[5], local_graph.vertices[6]]:
        assert resource.name.endswith("[\"bucket_a\"]") or resource.name.endswith("[\"bucket_b\"]")
        assert resource.id.endswith("[\"bucket_a\"]") or resource.id.endswith("[\"bucket_b\"]")
        config_name = list(resource.config['aws_s3_bucket'].keys())[0]
        assert config_name.endswith("[\"bucket_a\"]") or config_name.endswith("[\"bucket_b\"]")


def test_resources_flow():
    dir_name = 'foreach_examples/depend_resources'
    local_graph, _ = build_and_get_graph_by_path(dir_name, render_var=True)
    assert local_graph.vertices_by_block_type['variable'] == [1, 2]
    assert local_graph.vertices_by_block_type['resource'] == [0, 3]

    assert local_graph.vertices_block_name_map['variable'] == {'foreach_map': [1], 'test': [2]}
    assert local_graph.vertices_block_name_map['resource'] == {'aws_s3_bucket.foreach_map[\"bucket_a\"]': [0], 'aws_s3_bucket.foreach_map[\"bucket_b\"]': [3]}

    assert local_graph.edges[0].dest == 2
    assert local_graph.edges[0].origin == 0
    assert local_graph.edges[0].label == 'location'

    assert local_graph.edges[1].dest == 2
    assert local_graph.edges[1].origin == 3
    assert local_graph.edges[1].label == 'location'

    assert len(local_graph.vertices) == 4
    resources = [ver for ver in local_graph.vertices if ver.block_type == 'resource']
    assert len(resources) == 2

    resource_a_name = 'aws_s3_bucket.foreach_map[\"bucket_a\"]'
    assert resources[0].name == resource_a_name
    assert resources[0].id == resource_a_name
    assert resources[0].attributes.get('__address__') == resource_a_name
    assert resources[0].config.get('aws_s3_bucket').get('foreach_map[\"bucket_a\"]').get('__address__') == resource_a_name
    assert resources[0].attributes.get('location') == ["test"]
    assert resources[0].attributes.get('name') == ["bucket_a"]
    assert resources[0].attributes.get('region') == ["us-west-2"]
    assert list(resources[0].config.get('aws_s3_bucket').keys())[0] == 'foreach_map[\"bucket_a\"]'


@mock.patch.dict(os.environ, {"CHECKOV_NEW_TF_PARSER": "False"})
def test_tf_definitions_and_breadcrumbs():
    from checkov.terraform.graph_builder.graph_to_tf_definitions import convert_graph_vertices_to_tf_definitions
    dir_name = 'foreach_examples/depend_resources'
    local_graph, _ = build_and_get_graph_by_path(dir_name, render_var=True)
    tf_definitions, breadcrumbs = convert_graph_vertices_to_tf_definitions(local_graph.vertices, dir_name)
    expected_data = load_expected_data('expected_data_foreach.json')
    tf_definitions_to_check = {}
    for path, res in tf_definitions.items():
        path_list = path.file_path.split('/')[-2:]
        real_path = os.path.join(path_list[0], path_list[1])
        tf_definitions_to_check[real_path] = tf_definitions[path]
    assert_object_equal(tf_definitions_to_check, expected_data['tf_definitions'])

    expected_breadcrumbs = expected_data['breadcrumbs']
    assert len(breadcrumbs) == len(expected_breadcrumbs)
    assert len(breadcrumbs[list(breadcrumbs.keys())[0]]) == len(expected_breadcrumbs[list(expected_breadcrumbs.keys())[0]])
    resource_vertices = [vertex for vertex in local_graph.vertices if vertex.block_type == 'resource']
    for resource_vertex in resource_vertices:
        assert len(resource_vertex.foreach_attrs) == 2

    for name in ['["bucket_a"]', '["bucket_b"]']:
        assert f'aws_s3_bucket.foreach_map{name}' in breadcrumbs[list(breadcrumbs.keys())[0]]

        location_var = 'location'
        assert list(breadcrumbs[list(breadcrumbs.keys())[0]][f'aws_s3_bucket.foreach_map{name}'].keys()) == [location_var]
        assert list(expected_breadcrumbs[list(expected_breadcrumbs.keys())[0]][f'aws_s3_bucket.foreach_map{name}'].keys()) == [location_var]

        assert breadcrumbs[list(breadcrumbs.keys())[0]][f'aws_s3_bucket.foreach_map{name}'][location_var][0]['type'] == 'variable'
        assert expected_breadcrumbs[list(expected_breadcrumbs.keys())[0]][f'aws_s3_bucket.foreach_map{name}'][location_var][0]['type'] == 'variable'

        assert breadcrumbs[list(breadcrumbs.keys())[0]][f'aws_s3_bucket.foreach_map{name}'][location_var][0]['name'] == 'test'
        assert expected_breadcrumbs[list(expected_breadcrumbs.keys())[0]][f'aws_s3_bucket.foreach_map{name}'][location_var][0]['name'] == 'test'

        assert breadcrumbs[list(breadcrumbs.keys())[0]][f'aws_s3_bucket.foreach_map{name}'][location_var][0]['path'].endswith('depend_resources/variable.tf')
        assert expected_breadcrumbs[list(expected_breadcrumbs.keys())[0]][f'aws_s3_bucket.foreach_map{name}'][location_var][0]['path'].endswith('depend_resources/variable.tf')


@pytest.mark.parametrize(
    "attrs,k_v_to_change,expected_attrs,expected_res",
    [
        ({"test_key": ["${test_val}"]}, {"test_val": "new_val"}, {"test_key": ["new_val"]}, ['test_key']),
        ({"test_key": ["${test}"]}, {"test_val": "new_val"}, {"test_key": ["${test}"]}, []),
        ({"test_key": ["${test_val} ${test_val}"]}, {"test_val": "new_val"}, {"test_key": ["new_val new_val"]}, ['test_key']),
        ({"test_key": {"nested_key": ["${test_val}"]}}, {"test_val": "new_val"}, {"test_key": {"nested_key": ["new_val"]}}, ['test_key.nested_key']),
        ({"test_key": ["${test_val} test_val"]}, {"test_val": "new_val"}, {"test_key": ["new_val new_val"]}, ['test_key']),
        ({"test_key": ["${test_val}"]}, {"test_val": 123}, {"test_key": [123]}, ['test_key']),
        ({"test_key": ["${test_val}"]}, {"test_val": True}, {"test_key": [True]}, ['test_key']),
        ({"test_key": {"a": "${test_val}"}}, {"test_val": "new_val"}, {"test_key": {"a": "new_val"}}, ['test_key.a']),
        ({"test_key": {"a": {"b": "${test_val}"}}}, {"test_val": "new_val"}, {"test_key": {"a": {"b": "new_val"}}}, ['test_key.a.b']),
        ({'ports': '${each.value.port}', 'protocol': 'tcp'}, {'each.value': {'name': 'name-open-ssh', 'port': '22', 'range': '0.0.0.0/0', 'tag': 'allow-ssh'}}, {'ports': '22', 'protocol': 'tcp'}, ['ports']),
        (
                {"tags": ["${try(merge(var.tags,{'product_owner': '${each.value.product_owner}'}),var.tags,{'git_commit': 'aaaaa', 'git_file': 'main.tf'})}"]},
                {'each.value': {'name': 'security', 'product_owner': 'barak@gmail.com'}, 'each.key': 'security'},
                {"tags": ["${try(merge(var.tags,{'product_owner': 'barak@gmail.com'}),var.tags,{'git_commit': 'aaaaa', 'git_file': 'main.tf'})}"]},
                ["tags"]
        )
    ]
)
def test_update_attrs(attrs, k_v_to_change, expected_attrs, expected_res):
    local_graph = build_and_get_graph_by_path('')[0]
    foreach_handler = ForeachAbstractHandler(local_graph)
    res = foreach_handler._update_attributes(attrs, k_v_to_change)
    assert attrs == expected_attrs
    assert res == expected_res


@mock.patch.dict(os.environ, {"CHECKOV_ENABLE_MODULES_FOREACH_HANDLING": "True"})
def test_new_tf_parser_with_foreach_modules(checkov_source_path):
    dir_name = 'parser_dup_nested'
    local_graph, _ = build_and_get_graph_by_path(dir_name, render_var=True)
    tf_definitions, _ = convert_graph_vertices_to_tf_definitions(vertices=local_graph.vertices, root_folder=dir_name)

    assert len(tf_definitions.keys()) == 14
    assert len([block for block in local_graph.vertices if block.block_type == 'resource']) == 8
    assert len([block for block in local_graph.vertices if block.block_type == 'module']) == 12

    assert len(local_graph.vertices) == 47
    assert len(local_graph.vertices_by_module_dependency) == 13

    assert local_graph.vertices_by_module_dependency[None]['module'] == [0, 1, 25, 36]

    first_module_vertex = local_graph.vertices[0]
    assert first_module_vertex.name == 's3_module["a"]' and first_module_vertex.for_each_index == 'a'

    second_module_vertex = local_graph.vertices[1]
    assert second_module_vertex.name == 's3_module2[0]' and second_module_vertex.for_each_index == 0

    twenty_fifth_module_vertex = local_graph.vertices[25]
    assert twenty_fifth_module_vertex.name == 's3_module["b"]' and twenty_fifth_module_vertex.for_each_index == 'b'

    thrirty_six_module_vertex = local_graph.vertices[36]
    assert thrirty_six_module_vertex.name == 's3_module2[1]' and thrirty_six_module_vertex.for_each_index == 1

    assert local_graph.vertices[26].source_module == {25}
    assert local_graph.vertices[37].source_module == {36}

    # check foreach_idx is updated correctly
    first_key = list(tf_definitions.keys())[0]
    first_value = tf_definitions[first_key]

    first_tf_module = first_value['module'][0]['s3_module["a"]']['__resolved__'][0]
    second_tf_module = first_value['module'][1]['s3_module2[0]']['__resolved__'][0]
    third_tf_module = first_value['module'][2]['s3_module["b"]']['__resolved__'][0]
    fourth_tf_module = first_value['module'][3]['s3_module2[1]']['__resolved__'][0]
    assert first_tf_module in tf_definitions
    assert second_tf_module in tf_definitions
    assert third_tf_module in tf_definitions
    assert fourth_tf_module in tf_definitions

    first_nested_module = tf_definitions[first_tf_module]
    second_nested_module = tf_definitions[second_tf_module]
    third_nested_module = tf_definitions[third_tf_module]
    fourth_nested_module = tf_definitions[fourth_tf_module]

    assert len(tf_definitions[first_nested_module['module'][0]['inner_s3_module']['__resolved__'][0]]['resource']) == 1
    assert len(tf_definitions[first_nested_module['module'][1]['inner_s3_module2']['__resolved__'][0]]['resource']) == 1
    assert len(tf_definitions[second_nested_module['module'][0]['inner_s3_module']['__resolved__'][0]]['resource']) == 1
    assert len(tf_definitions[second_nested_module['module'][1]['inner_s3_module2']['__resolved__'][0]]['resource']) == 1
    assert len(tf_definitions[third_nested_module['module'][0]['inner_s3_module']['__resolved__'][0]]['resource']) == 1
    assert len(tf_definitions[third_nested_module['module'][1]['inner_s3_module2']['__resolved__'][0]]['resource']) == 1
    assert len(tf_definitions[fourth_nested_module['module'][0]['inner_s3_module']['__resolved__'][0]]['resource']) == 1
    assert len(tf_definitions[fourth_nested_module['module'][1]['inner_s3_module2']['__resolved__'][0]]['resource']) == 1

    assert first_tf_module.file_path == os.path.join(checkov_source_path, 'tests/terraform/graph/variable_rendering/resources/parser_dup_nested/module/main.tf')

    first_source_module = first_tf_module.tf_source_modules
    assert first_source_module.name == 's3_module'
    assert first_source_module.nested_tf_module is None
    assert first_source_module.foreach_idx == 'a'


@mock.patch.dict(os.environ, {"CHECKOV_ENABLE_MODULES_FOREACH_HANDLING": "True"})
def test_tf_definitions_for_foreach_on_modules(checkov_source_path):
    dir_name_and_definitions_path = [
        ('parser_dup_nested', 'expected_foreach_modules_tf_definitions.json'),
        ('foreach_module_dup_foreach', 'expected_foreach_module_dup_foreach.json')
    ]
    for dir_name, definitions_path in dir_name_and_definitions_path:
        local_graph, _ = build_and_get_graph_by_path(dir_name, render_var=True)
        tf_definitions, _ = convert_graph_vertices_to_tf_definitions(vertices=local_graph.vertices, root_folder=dir_name)

        file_path = os.path.join(os.path.dirname(__file__), definitions_path)
        with open(file_path, 'r') as f:
            expected_data = json.load(f, object_hook=object_hook)

        tf_definitions_json = json.dumps(tf_definitions, cls=CustomJSONEncoder)
        tf_definitions_json = tf_definitions_json.replace(checkov_source_path, '...')
        tf_definitions_after_handling_checkov_source = json.loads(tf_definitions_json, object_hook=object_hook)
        assert tf_definitions_after_handling_checkov_source == expected_data


@mock.patch.dict(os.environ, {"CHECKOV_ENABLE_MODULES_FOREACH_HANDLING": "True"})
def test_foreach_module_in_second_level_module(checkov_source_path):
    dir_name = 'foreach_module'
    graph, _ = build_and_get_graph_by_path(dir_name, render_var=True)
    tf_definitions, _ = convert_graph_vertices_to_tf_definitions(vertices=graph.vertices, root_folder=dir_name)

    assert len([block for block in graph.vertices if block.block_type == 'module']) == 10
    assert len([block for block in graph.vertices if block.block_type == 'resource']) == 8
    assert len(tf_definitions.keys()) == 11


@mock.patch.dict(os.environ, {"CHECKOV_ENABLE_MODULES_FOREACH_HANDLING": "True"})
def test_foreach_module_in_both_levels_module(checkov_source_path):
    dir_name = 'foreach_module_dup_foreach'
    graph, _ = build_and_get_graph_by_path(dir_name, render_var=True)
    tf_definitions, _ = convert_graph_vertices_to_tf_definitions(vertices=graph.vertices, root_folder=dir_name)

    resources = [block for block in graph.vertices if block.block_type == 'resource']
    locals = [block for block in graph.vertices if block.block_type == 'locals']
    vars = [block for block in graph.vertices if block.block_type == 'variable']
    modules = [block for block in graph.vertices if block.block_type == 'module']

    assert len(modules) == 20
    assert len(resources) == 16
    assert len(tf_definitions.keys()) == 22

    for resource in resources:
        assert resource.source_module_object.foreach_idx is not None

    for local in locals:
        assert local.source_module_object.foreach_idx is not None

    for var in vars:
        if var.source_module_object:
            assert var.source_module_object.foreach_idx is not None

    for module in modules:
        if module.source_module_object:
            assert module.source_module_object.foreach_idx is not None


@mock.patch.dict(os.environ, {"CHECKOV_ENABLE_MODULES_FOREACH_HANDLING": "True"})
def test_foreach_module_and_resource(checkov_source_path):
    dir_name = 'foreach_module_and_resource'
    graph, _ = build_and_get_graph_by_path(dir_name, render_var=True)
    tf_definitions, _ = convert_graph_vertices_to_tf_definitions(vertices=graph.vertices, root_folder=dir_name)

    assert len([block for block in graph.vertices if block.block_type == 'module']) == 2
    assert len([block for block in graph.vertices if block.block_type == 'resource']) == 4
    assert len(tf_definitions.keys()) == 3

    assert graph.vertices[2].config['aws_s3_bucket_public_access_block']['var_bucket["a"]']['__address__'] == 'module.s3_module["a"].aws_s3_bucket_public_access_block.var_bucket["a"]'
    assert graph.vertices[6].config['aws_s3_bucket_public_access_block']['var_bucket["a"]']['__address__'] == 'module.s3_module["b"].aws_s3_bucket_public_access_block.var_bucket["a"]'
    assert graph.vertices[8].config['aws_s3_bucket_public_access_block']['var_bucket["b"]']['__address__'] == 'module.s3_module["a"].aws_s3_bucket_public_access_block.var_bucket["b"]'
    assert graph.vertices[9].config['aws_s3_bucket_public_access_block']['var_bucket["b"]']['__address__'] == 'module.s3_module["b"].aws_s3_bucket_public_access_block.var_bucket["b"]'


@mock.patch.dict(os.environ, {"CHECKOV_ENABLE_MODULES_FOREACH_HANDLING": "True", "CHECKOV_ENABLE_DATAS_FOREACH_HANDLING": "True"})
def test_foreach_data(checkov_source_path):
    dir_name = 'data_simple'
    graph, _ = build_and_get_graph_by_path(dir_name, render_var=True)
    tf_definitions, _ = convert_graph_vertices_to_tf_definitions(vertices=graph.vertices, root_folder=dir_name)

    assert len([block for block in graph.vertices if block.block_type == 'data']) == 6
    assert len(tf_definitions[list(tf_definitions.keys())[0]]['data']) == 6

    data_vertices_names = [block.name for block in graph.vertices if block.block_type == 'data']
    assert 'aws_s3_bucket.data_list["b"]' in data_vertices_names
    assert 'aws_s3_bucket.data_dict["key1"]' in data_vertices_names
    assert 'aws_s3_bucket.data_count[0]' in data_vertices_names
    assert 'aws_s3_bucket.data_list["a"]' in data_vertices_names
    assert 'aws_s3_bucket.data_dict["key2"]' in data_vertices_names
    assert 'aws_s3_bucket.data_count[1]' in data_vertices_names


@mock.patch.dict(os.environ, {"CHECKOV_ENABLE_MODULES_FOREACH_HANDLING": "True", "CHECKOV_ENABLE_DATAS_FOREACH_HANDLING": "True"})
def test_foreach_data_with_resource(checkov_source_path):
    dir_name = 'data_with_resource'
    graph, _ = build_and_get_graph_by_path(dir_name, render_var=True)
    tf_definitions, _ = convert_graph_vertices_to_tf_definitions(vertices=graph.vertices, root_folder=dir_name)

    assert len([block for block in graph.vertices if block.block_type == 'data']) == 5
    assert len(tf_definitions[list(tf_definitions.keys())[0]]['data']) == 5

    data_vertices_names = [block.name for block in graph.vertices if block.block_type == 'data']
    assert 'aws_s3_bucket.data_dict["key1"]' in data_vertices_names
    assert 'aws_s3_bucket.data_count[0]' in data_vertices_names
    assert 'aws_s3_bucket.data_dict["key2"]' in data_vertices_names
    assert 'aws_s3_bucket.data_count[1]' in data_vertices_names

    assert graph.vertices[0].attributes['bucket'] == graph.vertices[3].attributes['bucket']
    assert graph.vertices[1].attributes['bucket'] == graph.vertices[4].attributes['bucket']
    assert graph.vertices[8].attributes['bucket'] == graph.vertices[10].attributes['bucket']
    assert graph.vertices[9].attributes['bucket'] == graph.vertices[11].attributes['bucket']


@mock.patch.dict(os.environ, {"CHECKOV_ENABLE_MODULES_FOREACH_HANDLING": "True"})
def test_foreach_module_with_more_than_two_resources(checkov_source_path):
    dir_name = 'foreach_module_with_more_than_two_resources'
    graph, _ = build_and_get_graph_by_path(dir_name, render_var=True)
    tf_definitions, _ = convert_graph_vertices_to_tf_definitions(vertices=graph.vertices, root_folder=dir_name)

    assert len([block for block in graph.vertices if block.block_type == 'module']) == 16
    assert len([block for block in graph.vertices if block.block_type == 'resource']) == 14
    assert len(tf_definitions.keys()) == 17


@pytest.mark.parametrize(
    "statement,expected",
    [
        ([{'main'}], True),
        (["${toset(['bucket_a', 'bucket_b'])}"], True),
        ({'key1': '${var.a}', 'key2': '${var.b}'}, True),
        ({'key2': '${var.b}', 'var.a': '${var.a}'}, False),
        ('${var.a}', False),
        ('bana', True)
    ]
)
def test__is_static_foreach_statement(statement, expected):
    abstract_handler = ForeachAbstractHandler(None)
    assert abstract_handler._is_static_foreach_statement(statement) == expected


@mock.patch.dict(os.environ, {"CHECKOV_ENABLE_MODULES_FOREACH_HANDLING": "True"})
def test_foreach_with_lookup():
    dir_name = 'foreach_examples/foreach_lookup'
    graph, _ = build_and_get_graph_by_path(dir_name, render_var=True)
    assert graph.vertices[0].attributes.get('uniform_bucket_level_access') == [True]
    assert graph.vertices[1].attributes.get('uniform_bucket_level_access') == [True]


@mock.patch.dict(os.environ, {"CHECKOV_ENABLE_MODULES_FOREACH_HANDLING": "True"})
def test_foreach_large_count_with_nested_module(checkov_source_path):
    dir_name = 'os_example_large_count_with_nested_module'
    graph, _ = build_and_get_graph_by_path(dir_name, render_var=True)
    assert len(graph.vertices) == 85


def test__get_tf_module_with_no_foreach():
    module = TFModule(name='1', path='1', foreach_idx='1',
                      nested_tf_module=TFModule(name='2', path='2', foreach_idx='2', nested_tf_module=None))
    result = ForeachModuleHandler._get_tf_module_with_no_foreach(module)
    assert result == TFModule(name='1', path='1', foreach_idx=None,
                      nested_tf_module=TFModule(name='2', path='2', foreach_idx=None, nested_tf_module=None))


def test__get_module_with_only_relevant_foreach_idx():
    module = TFModule(name='1', path='1', foreach_idx='1',
                      nested_tf_module=TFModule(name='2', path='2', foreach_idx='2',
                                                nested_tf_module=TFModule(name='3', path='3', foreach_idx='3',
                                                                          nested_tf_module=None)
                                                )
                      )
    original_key = TFModule(name='2', path='2', foreach_idx='2',
                            nested_tf_module=TFModule(name='3', path='3', foreach_idx='3', nested_tf_module=None))
    result = ForeachModuleHandler._get_module_with_only_relevant_foreach_idx('test', original_key, module)
    assert result == TFModule(name='1', path='1', foreach_idx='1',
                              nested_tf_module=TFModule(name='2', path='2', foreach_idx='test',
                                                        nested_tf_module=TFModule(name='3', path='3', foreach_idx='3',
                                                                                  nested_tf_module=None)
                                                        )
                              )

def test_nested_foreach_with_variable_reference():
    """
    Here we test that a nested foreach loop based on module locals is correctly rendered in the Terraform graph.
    """
    resources_by_group_local_var = 2
    resources_by_files_local_var = 2

    dir_name = 'foreach_examples/nested_foreach_based_on_module_locals'
    graph = build_and_get_graph_by_path(dir_name)[0]
    graph_resources_filter = filter(lambda blk: blk.block_type == 'resource', graph.vertices)
    graph_resources_created = list(map(lambda rsrc: rsrc.attributes['__address__'], graph_resources_filter))

    assert len(graph_resources_created) is (resources_by_group_local_var * resources_by_files_local_var)
    assert graph_resources_created == ['module.files["blue"].aws_s3_bucket_object.this_file["test1"]',
                                       'module.files["green"].aws_s3_bucket_object.this_file["test1"]',
                                       'module.files["blue"].aws_s3_bucket_object.this_file["test2"]',
                                       'module.files["green"].aws_s3_bucket_object.this_file["test2"]']


def test_double_nested_foreach_with_variable_reference():
    """
    Here we test that a 2 level nested foreach loop based on module local vars is correctly rendered in the Terraform graph.

    In this test we have 2 x level1 modules (green, blue) each has 2 level2 modules (test1.txt, test2.txt)
    and 2 resources for each (test3.txt, test4.txt).
    So (2 x level1) -> (2 x level2) -> (2 x aws_s3_bucket resource).

    The unique use case is that the for_each attributes depends on the main module's local variables.
    """
    dir_name = 'foreach_examples/module_foreach_module_foreach_resource_foreach'
    graph = build_and_get_graph_by_path(dir_name)[0]

    graph_modules_filter = filter(lambda blk: blk.block_type == 'module', graph.vertices)
    graph_modules_created = list(map(lambda rsrc: rsrc.attributes['__address__'], graph_modules_filter))

    graph_resources_filter = filter(lambda blk: blk.block_type == 'resource', graph.vertices)
    graph_resources_created = list(map(lambda rsrc: rsrc.attributes['__address__'], graph_resources_filter))

    assert len(graph_modules_created) is 6    # 2 level1 modules, each has 2 level2 modules (total of 2 + 2*2 = 6)
    assert len(graph_resources_created) is 8  # 4 level2 modules, each has 2 resources (total of 2*2*2 = 8)

    assert graph_resources_created == ['module.level1["blue"].module.level2["test1.txt"].aws_s3_bucket_object.this_file["test3.txt"]',
                                       'module.level1["green"].module.level2["test1.txt"].aws_s3_bucket_object.this_file["test3.txt"]',
                                       'module.level1["blue"].module.level2["test2.txt"].aws_s3_bucket_object.this_file["test3.txt"]',
                                       'module.level1["green"].module.level2["test2.txt"].aws_s3_bucket_object.this_file["test3.txt"]',
                                       'module.level1["blue"].module.level2["test1.txt"].aws_s3_bucket_object.this_file["test4.txt"]',
                                       'module.level1["green"].module.level2["test1.txt"].aws_s3_bucket_object.this_file["test4.txt"]',
                                       'module.level1["blue"].module.level2["test2.txt"].aws_s3_bucket_object.this_file["test4.txt"]',
                                       'module.level1["green"].module.level2["test2.txt"].aws_s3_bucket_object.this_file["test4.txt"]']


def test_double_nested_foreach_and_count_with_variable_reference():
    """
    Here we test that a 2 level nested foreach loop and count based on module locals is correctly rendered in the Terraform graph.
    In this test we have 2 x level1 modules (green, blue) each has 2 level2 modules (test1.txt, test2.txt)
    and 2 resources for each (count of 2).
    So (2 x level1) -> (2 x level2) -> (2 x aws_s3_bucket resource: count = 2).

    The unique use case is that the count and for_each attributes (multiple levels) depends on the main module's local variables.
    """
    dir_name = 'count_examples/module_foreach_module_foreach_resource_count'
    graph = build_and_get_graph_by_path(dir_name)[0]

    graph_modules_filter = filter(lambda blk: blk.block_type == 'module', graph.vertices)
    graph_modules_created = list(map(lambda rsrc: rsrc.attributes['__address__'], graph_modules_filter))

    graph_resources_filter = filter(lambda blk: blk.block_type == 'resource', graph.vertices)
    graph_resources_created = list(map(lambda rsrc: rsrc.attributes['__address__'], graph_resources_filter))

    assert len(graph_modules_created) is 6    # 2 level1 modules, each has 2 level2 modules (total of 2 + 2*2 = 6)
    assert len(graph_resources_created) is 8  # 4 level2 modules, each has 2 resources (total of 2*2*2 = 8)

    assert graph_resources_created == ['module.level1["blue"].module.level2["test1.txt"].aws_s3_bucket_object.this_file[0]',
                                       'module.level1["green"].module.level2["test1.txt"].aws_s3_bucket_object.this_file[0]',
                                       'module.level1["blue"].module.level2["test2.txt"].aws_s3_bucket_object.this_file[0]',
                                       'module.level1["green"].module.level2["test2.txt"].aws_s3_bucket_object.this_file[0]',
                                       'module.level1["blue"].module.level2["test1.txt"].aws_s3_bucket_object.this_file[1]',
                                       'module.level1["green"].module.level2["test1.txt"].aws_s3_bucket_object.this_file[1]',
                                       'module.level1["blue"].module.level2["test2.txt"].aws_s3_bucket_object.this_file[1]',
                                       'module.level1["green"].module.level2["test2.txt"].aws_s3_bucket_object.this_file[1]']


@mock.patch.object(env_vars_config, "RAW_TF_IN_GRAPH_ENV", "True")
def test_foreach_renderer_with_raw_asset():
    dir_name = 'foreach_examples/foreach_dup_resources'
    local_graph = build_and_get_graph_by_path(dir_name)[0]
    foreach_builder = ForeachBuilder(local_graph)
    foreach_builder._module_handler.local_graph.enable_foreach_handling = True
    assert len(local_graph.vertices) == 9
    for resource in [local_graph.vertices[0], local_graph.vertices[1], local_graph.vertices[5], local_graph.vertices[7]]:
        assert resource.name.endswith("[\"bucket_a\"]") or resource.name.endswith("[\"bucket_b\"]")
        assert resource.id.endswith("[\"bucket_a\"]") or resource.id.endswith("[\"bucket_b\"]")
        config_name = list(resource.config['aws_s3_bucket'].keys())[0]
        assert config_name.endswith("[\"bucket_a\"]") or config_name.endswith("[\"bucket_b\"]")
    for edge in [local_graph.edges[1], local_graph.edges[2], local_graph.edges[4], local_graph.edges[5]]:
        assert edge.label == 'virtual_resource'
    for resource in [local_graph.vertices[6], local_graph.vertices[8]]:
        assert len(resource.config[CustomAttributes.VIRTUAL_RESOURCES]) == 2
        for virtual_resource in resource.config[CustomAttributes.VIRTUAL_RESOURCES]:
            assert virtual_resource.endswith("[\"bucket_a\"]") or virtual_resource.endswith("[\"bucket_b\"]")






import unittest

from checkov.terraform.checks.resource.aws.CloudfrontDistributionEncryption import check
from checkov.common.models.enums import CheckResult


class TestCloudfrontDistributionEncryption(unittest.TestCase):

    def test_failure(self):
        resource_conf = {'origin': [
            {'domain_name': ['${aws_s3_bucket.b.bucket_regional_domain_name}'], 'origin_id': ['${local.s3_origin_id}'],
             's3_origin_config': [{'origin_access_identity': ['origin-access-identity/cloudfront/ABCDEFG1234567']}]}],
            'enabled': [True], 'is_ipv6_enabled': [True], 'comment': ['Some comment'],
            'default_root_object': ['index.html'], 'logging_config': [
                {'include_cookies': [False], 'bucket': ['mylogs.s3.amazonaws.com'], 'prefix': ['myprefix']}],
            'aliases': [['mysite.example.com', 'yoursite.example.com']], 'ordered_cache_behavior': [
                {'path_pattern': ['/content/immutable/*'], 'allowed_methods': [['GET', 'HEAD', 'OPTIONS']],
                 'cached_methods': [['GET', 'HEAD', 'OPTIONS']], 'target_origin_id': ['${local.s3_origin_id}'],
                 'forwarded_values': [
                     {'query_string': [False], 'headers': [['Origin']], 'cookies': [{'forward': ['none']}]}],
                 'min_ttl': [0], 'default_ttl': [86400], 'max_ttl': [31536000], 'compress': [True],
                 'viewer_protocol_policy': ['redirect-to-https']},
                {'path_pattern': ['/content/*'], 'allowed_methods': [['GET', 'HEAD', 'OPTIONS']],
                 'cached_methods': [['GET', 'HEAD']], 'target_origin_id': ['${local.s3_origin_id}'],
                 'forwarded_values': [{'query_string': [False], 'cookies': [{'forward': ['none']}]}], 'min_ttl': [0],
                 'default_ttl': [3600], 'max_ttl': [86400], 'compress': [True],
                 'viewer_protocol_policy': ['redirect-to-https']}], 'price_class': ['PriceClass_200'], 'restrictions': [
                {'geo_restriction': [{'restriction_type': ['whitelist'], 'locations': [['US', 'CA', 'GB', 'DE']]}]}],
            'viewer_certificate': [{'cloudfront_default_certificate': [True]}], 'default_cache_behavior': [
                {'allowed_methods': [['DELETE', 'GET', 'HEAD', 'OPTIONS', 'PATCH', 'POST', 'PUT']],
                 'cached_methods': [['GET', 'HEAD']], 'target_origin_id': ['${local.s3_origin_id}'],
                 'forwarded_values': [{'query_string': [False], 'cookies': [{'forward': ['none']}]}],
                 'viewer_protocol_policy': ['allow-all'], 'min_ttl': [0], 'default_ttl': [3600], 'max_ttl': [86400]}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {'origin': [
            {'domain_name': ['${aws_s3_bucket.b.bucket_regional_domain_name}'], 'origin_id': ['${local.s3_origin_id}'],
             's3_origin_config': [{'origin_access_identity': ['origin-access-identity/cloudfront/ABCDEFG1234567']}]}],
            'enabled': [True], 'is_ipv6_enabled': [True], 'comment': ['Some comment'],
            'default_root_object': ['index.html'], 'logging_config': [
                {'include_cookies': [False], 'bucket': ['mylogs.s3.amazonaws.com'], 'prefix': ['myprefix']}],
            'aliases': [['mysite.example.com', 'yoursite.example.com']], 'ordered_cache_behavior': [
                {'path_pattern': ['/content/immutable/*'], 'allowed_methods': [['GET', 'HEAD', 'OPTIONS']],
                 'cached_methods': [['GET', 'HEAD', 'OPTIONS']], 'target_origin_id': ['${local.s3_origin_id}'],
                 'forwarded_values': [
                     {'query_string': [False], 'headers': [['Origin']], 'cookies': [{'forward': ['none']}]}],
                 'min_ttl': [0], 'default_ttl': [86400], 'max_ttl': [31536000], 'compress': [True],
                 'viewer_protocol_policy': ['redirect-to-https']},
                {'path_pattern': ['/content/*'], 'allowed_methods': [['GET', 'HEAD', 'OPTIONS']],
                 'cached_methods': [['GET', 'HEAD']], 'target_origin_id': ['${local.s3_origin_id}'],
                 'forwarded_values': [{'query_string': [False], 'cookies': [{'forward': ['none']}]}], 'min_ttl': [0],
                 'default_ttl': [3600], 'max_ttl': [86400], 'compress': [True],
                 'viewer_protocol_policy': ['redirect-to-https']}], 'price_class': ['PriceClass_200'], 'restrictions': [
                {'geo_restriction': [{'restriction_type': ['whitelist'], 'locations': [['US', 'CA', 'GB', 'DE']]}]}],
            'viewer_certificate': [{'cloudfront_default_certificate': [True]}], 'default_cache_behavior': [
                {'allowed_methods': [['DELETE', 'GET', 'HEAD', 'OPTIONS', 'PATCH', 'POST', 'PUT']],
                 'cached_methods': [['GET', 'HEAD']], 'target_origin_id': ['${local.s3_origin_id}'],
                 'forwarded_values': [{'query_string': [False], 'cookies': [{'forward': ['none']}]}],
                 'viewer_protocol_policy': ['redirect-to-https'], 'min_ttl': [0], 'default_ttl': [3600],
                 'max_ttl': [86400]}]}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import os

from parameterized import parameterized_class

from tests.graph_utils.utils import PARAMETERIZED_GRAPH_FRAMEWORKS
from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class TestRangeNotIncludesSolver(TestBaseSolver):
    def setUp(self):
        self.checks_dir = TEST_DIRNAME
        super(TestRangeNotIncludesSolver, self).setUp()

    def test_range_not_includes_int_solver(self):
        root_folder = 'resources'
        check_id = "RangeNotIncludesInt"
        should_fail = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7']
        should_pass = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail4', 'test.fail5', 'test.fail6', 'test.fail7',
                       'test.fail8', 'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_range_not_includes_string_solver(self):
        root_folder = 'resources'
        check_id = "RangeNotIncludesString"
        should_fail = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7']
        should_pass = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail4', 'test.fail5', 'test.fail6', 'test.fail7',
                       'test.fail8', 'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_range_not_includes_int_jsonpath_solver(self):
        root_folder = 'resources'
        check_id = "JsonPathRangeNotIncludesInt"
        should_fail = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7']
        should_pass = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail4', 'test.fail5', 'test.fail6', 'test.fail7',
                       'test.fail8', 'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_range_not_includes_string_jsonpath_solver(self):
        root_folder = 'resources'
        check_id = "JsonPathRangeNotIncludesString"
        should_fail = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7']
        should_pass = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail4', 'test.fail5', 'test.fail6', 'test.fail7',
                       'test.fail8', 'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_range_not_includes_list_solver(self):
        root_folder = 'resources'
        check_id = "RangeNotIncludesList"
        should_fail = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7']
        should_pass = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail4', 'test.fail5', 'test.fail6', 'test.fail7',
                       'test.fail8', 'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_range_not_includes_list_jsonpath_solver(self):
        root_folder = 'resources'
        check_id = "JsonPathRangeNotIncludesList"
        should_fail = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7']
        should_pass = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail4', 'test.fail5', 'test.fail6', 'test.fail7',
                       'test.fail8', 'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_range_not_includes_list_w_list_solver(self):
        root_folder = 'resources'
        check_id = "RangeNotIncludesListWRange"
        should_fail = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7',
                       'test.fail4']
        should_pass = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail5', 'test.fail6', 'test.fail7', 'test.fail8',
                       'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_range_not_includes_list_w_list_jsonpath_solver(self):
        root_folder = 'resources'
        check_id = "JsonPathRangeNotIncludesListWRange"
        should_fail = ['test.pass1', 'test.pass2', 'test.pass3', 'test.pass4', 'test.pass5', 'test.pass6', 'test.pass7',
                       'test.fail4']
        should_pass = ['test.fail1', 'test.fail2', 'test.fail3', 'test.fail5', 'test.fail6', 'test.fail7', 'test.fail8',
                       'test.fail9']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.oidc_utils import gh_abusable_claims, gh_repo_regex, gh_sub_condition
from checkov.common.util.type_forcers import extract_policy_dict, force_list
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class GithubActionsOIDCTrustPolicyOnRole(BaseResourceCheck):
    """
    Resource-level mirror of CKV_AWS_358 (GithubActionsOIDCTrustPolicy).

    CKV_AWS_358 only inspects `data "aws_iam_policy_document"` blocks. This
    check applies the SAME unsafe-claim logic to `aws_iam_role` resources
    that define their trust policy inline via `assume_role_policy`.

    Logic is intentionally identical to CKV_AWS_358; only the parsing layer
    differs (JSON statement objects vs. HCL `statement{}` blocks). Shared
    constants (gh_sub_condition, gh_abusable_claims, gh_repo_regex) come
    from checkov.common.util.oidc_utils.
    """

    GH_OIDC_PROVIDER_SUBSTR = "oidc-provider/token.actions.githubusercontent.com"

    def __init__(self) -> None:
        name = (
            "Ensure AWS GitHub Actions OIDC authorization policies only allow "
            "safe claims and claim order on IAM role"
        )
        id = "CKV_AWS_393"
        supported_resources = ("aws_iam_role",)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        if not conf.get("assume_role_policy"):
            return CheckResult.PASSED

        try:
            policy = extract_policy_dict(conf["assume_role_policy"][0])
        except Exception:  # nosec
            return CheckResult.PASSED

        if not policy or "Statement" not in policy:
            return CheckResult.PASSED

        for statement in force_list(policy["Statement"]):
            if not isinstance(statement, dict):
                continue

            if not self._has_github_oidc_federated_principal(statement):
                # No GH-OIDC federated principal in this statement -> not our concern.
                # (Matches CKV_AWS_358 behavior: "if not found_federated_gh_oidc: return PASSED")
                return CheckResult.PASSED

            # Federated GH-OIDC principal MUST come with a Condition.
            condition = statement.get("Condition")
            if not condition or not isinstance(condition, dict):
                return CheckResult.FAILED

            verdict = self._evaluate_sub_conditions(condition)
            if verdict is not None:
                return verdict

            # Federated GH-OIDC principal present, conditions present, but none
            # contained a recognized `:sub` constraint -> unsafe.
            return CheckResult.FAILED

        return CheckResult.PASSED

    def _has_github_oidc_federated_principal(self, statement: dict[str, Any]) -> bool:
        principal = statement.get("Principal")
        if not isinstance(principal, dict):
            return False
        federated = principal.get("Federated")
        if federated is None:
            return False
        for identifier in force_list(federated):
            if isinstance(identifier, str) and self.GH_OIDC_PROVIDER_SUBSTR in identifier:
                return True
        return False

    def _evaluate_sub_conditions(self, condition: dict[str, Any]) -> CheckResult | None:
        """
        Walk Condition operators looking for the `:sub` claim constraint.

        Mirrors CKV_AWS_358's inner loop: returns FAILED on the first unsafe
        value found, PASSED on the first safe value found, or None if no
        `:sub` claim constraint exists in any operator (the caller then
        returns FAILED, matching the data check's "Found a federated GitHub
        user, but no restrictions" path).
        """
        for operator_values in condition.values():
            if not isinstance(operator_values, dict):
                continue
            for variable_name, values in operator_values.items():
                if not isinstance(variable_name, str) or not gh_sub_condition.match(variable_name):
                    continue
                for value in force_list(values):
                    if not isinstance(value, str):
                        continue
                    verdict = self._classify_sub_value(value)
                    if verdict is not None:
                        return verdict
        return None

    @staticmethod
    def _classify_sub_value(value: str) -> CheckResult | None:
        """
        Same four unsafe-pattern checks as CKV_AWS_358, in the same order.
        Returns FAILED for an unsafe value, PASSED for a safe value, or
        None if the value is malformed in a way the data check would also
        skip (defensive; should not normally occur for string values).
        """
        # 1. Bare wildcard: "sub": "*"
        if value == "*":
            return CheckResult.FAILED

        split_claims = value.split(":")

        # 2. Bare claim name with no value: "sub": "invalid"
        if len(split_claims) == 1:
            return CheckResult.FAILED

        # 3. Wildcard assertion on the first claim's value: "sub": "claim:*"
        if split_claims[1] == "*":
            return CheckResult.FAILED

        # 4. Abusable first claim: "sub": "workflow:..." / "environment:..." / etc.
        for abusable_claim in gh_abusable_claims:
            if split_claims[0].startswith(abusable_claim):
                return CheckResult.FAILED

        # 5. "repo:" prefix must be followed by an org/repo-shaped value.
        #    "repo:myOrg*" -> FAIL ; "repo:myOrg/*" -> PASS (per gh_repo_regex)
        if split_claims[0] == "repo" and not gh_repo_regex.match(split_claims[1]):
            return CheckResult.FAILED

        return CheckResult.PASSED

    def get_evaluated_keys(self) -> list[str]:
        return ["assume_role_policy"]


check = GithubActionsOIDCTrustPolicyOnRole()



from typing import Optional, Any, Dict
from collections.abc import Collection

import hcl2

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver
from checkov.common.util.consts import START_LINE, END_LINE


class IsEmptyAttributeSolver(BaseAttributeSolver):
    operator = Operators.IS_EMPTY  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        attr = vertex.get(attribute)  # type:ignore[arg-type]  # due to attribute can be None

        if isinstance(attr, (list, Collection)):
            if len(attr) == 0 \
                    or (len(attr) == 2 and START_LINE in attr and END_LINE in attr) \
                    or (len(attr) == 2 and hcl2.START_LINE in attr and hcl2.END_LINE in attr):
                return True

        return False



from __future__ import annotations

import logging
import os
import warnings
from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, List, Dict, Any, Iterable, TypeVar, Generic

from checkov.common.graph.graph_builder import Edge
from checkov.common.graph.graph_builder.utils import run_function_multithreaded
from checkov.common.graph.graph_builder.graph_components.block_types import BlockType


if TYPE_CHECKING:
    from checkov.common.graph.graph_builder.graph_components.blocks import Block  # noqa
    from checkov.common.graph.graph_builder.local_graph import LocalGraph  # noqa

_LocalGraph = TypeVar("_LocalGraph", bound="LocalGraph[Any]")


class VariableRenderer(ABC, Generic[_LocalGraph]):
    MAX_NUMBER_OF_LOOPS = 50

    def __init__(self, local_graph: _LocalGraph) -> None:
        warnings.filterwarnings("ignore", category=SyntaxWarning)
        self.local_graph = local_graph
        self.run_async = True if os.getenv("RENDER_VARIABLES_ASYNC") == "True" else False
        self.max_workers = int(os.getenv("RENDER_ASYNC_MAX_WORKERS", 50))
        self.duplicate_percent = int(os.getenv("RENDER_EDGES_DUPLICATE_PERCENT", 90))
        self.duplicate_iter_count = int(os.getenv("RENDER_EDGES_DUPLICATE_ITER_COUNT", 4))
        self.done_edges_by_origin_vertex: Dict[int, List[Edge]] = {}
        self.replace_cache: List[Dict[str, Any]] = [{}] * len(local_graph.vertices)
        self.vertices_index_to_render: List[int] = []

    def render_variables_from_local_graph(self) -> None:
        self._render_variables_from_edges()
        self._render_variables_from_vertices()

    def _render_variables_from_edges(self) -> None:
        end_vertices_indexes = self._get_initial_end_vertices()
        edges_to_render = self.local_graph.get_in_edges(end_vertices_indexes)
        if self.vertices_index_to_render:
            edges_to_render = self._remove_unrelated_edges(edges_to_render)

        end_vertices_indexes = set()
        loops = 0
        evaluated_edges_cache: list[list[Edge]] = [[], []]
        duplicates_count = 0

        while edges_to_render:
            evaluated_edges_two_iter_ago = evaluated_edges_cache[-2]
            intersection_edges = set(edges_to_render).intersection(evaluated_edges_two_iter_ago)
            match_percent = int((len(intersection_edges) / len(edges_to_render)) * 100)
            if match_percent > self.duplicate_percent:
                duplicates_count += 1
            if duplicates_count > self.duplicate_iter_count:
                logging.info(f"Reached too many edge duplications of {self.duplicate_percent}% for {self.duplicate_iter_count} iterations. breaking.")
                break
            evaluated_edges_cache.append(edges_to_render)
            logging.debug(f"evaluating {len(edges_to_render)} edges; loop_num={loops}")

            edges_groups = self.group_edges_by_origin_and_label(edges_to_render)

            self._evaluate_edge_groups(edges_groups)

            self._update_done_edges_by_origin_vertex(edges_to_render)

            self._update_end_vertices_indexes(edges_to_render, end_vertices_indexes)

            new_edges_to_render = self.local_graph.get_in_edges_deduped(end_vertices_indexes)

            edges_to_render = self.local_graph.sort_edged_by_dest_out_degree(
                new_edges_to_render - set(edges_to_render)
            )

            loops += 1
            if loops >= self.MAX_NUMBER_OF_LOOPS:
                logging.warning(f"Reached max ({self.MAX_NUMBER_OF_LOOPS}) graph edge evaluation loops, breaking.")
                break

        if self.vertices_index_to_render:
            return
        self.local_graph.update_vertices_configs()
        logging.debug("done evaluating edges")
        self.evaluate_non_rendered_values()
        logging.debug("done evaluate_non_rendered_values")

    def _get_initial_end_vertices(self) -> set[int]:
        return self.local_graph.get_vertices_with_degrees_conditions(
            out_degree_cond=lambda d: d == 0,
            in_degree_cond=lambda d: d > 0,
        )

    def _evaluate_edge_groups(self, edges_groups: list[list[Edge]]) -> None:
        if self.run_async:
            run_function_multithreaded(
                func=self._edge_evaluation_task,
                data=edges_groups,
                max_group_size=1,
                num_of_workers=self.max_workers,
            )
        else:
            for edge_group in edges_groups:
                self._edge_evaluation_task([edge_group])

    def _update_done_edges_by_origin_vertex(self, edges_to_render: list[Edge]) -> None:
        for edge in edges_to_render:
            origin = edge.origin
            self.done_edges_by_origin_vertex.setdefault(origin, []).append(edge)

    def _update_end_vertices_indexes(self, edges_to_render: list[Edge], end_vertices_indexes: set[int]) -> None:
        already_checked: set[int] = set()

        for edge in edges_to_render:
            origin_vertex_index = edge.origin

            # Only check each origin once
            if origin_vertex_index in already_checked:
                continue
            already_checked.add(origin_vertex_index)

            out_edges = set(self.local_graph.out_edges.get(origin_vertex_index, []))
            done_edges_for_origin = set(self.done_edges_by_origin_vertex.get(origin_vertex_index, []))

            if out_edges.issubset(done_edges_for_origin):
                end_vertices_indexes.add(origin_vertex_index)

    @abstractmethod
    def _render_variables_from_vertices(self) -> None:
        pass

    def _edge_evaluation_task(self, edges: List[List[Edge]]) -> List[Edge]:
        inner_edges = edges[0]
        self.evaluate_vertex_attribute_from_edge(inner_edges)
        return inner_edges

    def _remove_unrelated_edges(self, edges_to_render: List[Edge]) -> List[Edge]:
        new_edges_to_render = []
        for edge in edges_to_render:
            if not self.local_graph.vertices[edge.origin] == BlockType.RESOURCE or edge.origin not in self.vertices_index_to_render:
                new_edges_to_render.append(edge)
        return new_edges_to_render

    @abstractmethod
    def evaluate_vertex_attribute_from_edge(self, edge_list: List[Edge]) -> None:
        pass

    @staticmethod
    def group_edges_by_origin_and_label(edges: Iterable[Edge]) -> List[List[Edge]]:
        edge_groups: Dict[str, List[Edge]] = {}
        for edge in edges:
            edge_groups.setdefault(f"{edge.origin}{edge.label}", []).append(edge)
        return list(edge_groups.values())

    @abstractmethod
    def evaluate_non_rendered_values(self) -> None:
        pass



import os
import unittest
import sys
import platform

current_dir = os.path.dirname(os.path.realpath(__file__))
ext_modules_path = os.path.join(current_dir, "example_ext_private_modules", ".external_modules")


class TestCheckovExtModuleCloning(unittest.TestCase):

    def test_private_github_modules_api_key(self):
        if sys.version_info[1] == 8 and platform.system() == 'Linux' and False:
            expected_private_github_path = os.path.join(ext_modules_path, "github.com", "ckv-tests")
            expected_private_github_modules = [os.path.join(expected_private_github_path, "terraform-aws-iam-s3-user-private"),
                                               os.path.join(expected_private_github_path, "terraform-aws-s3-bucket-private")]
            for m in expected_private_github_modules:
                assert os.path.exists(m)

    def test_private_tfc_modules_api_key(self):
        if sys.version_info[1] == 8 and platform.system() == 'Linux' and False:
            expected_private_tfc_path = os.path.join(ext_modules_path, "app.terraform.io", "panw-bridgecrew")
            expected_private_tfc_modules = [
                os.path.join(expected_private_tfc_path, "iam-s3-user", "aws", "0.15.7"),
                os.path.join(expected_private_tfc_path, "s3-bucket1", "aws", "0.0.2")]
            for m in expected_private_tfc_modules:
                assert os.path.exists(m)



import os
import unittest

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.gitlab_ci.runner import Runner
from checkov.runner_filter import RunnerFilter
from checkov.gitlab_ci.checks.registry import registry


class TestRunnerValid(unittest.TestCase):

    def test_runner(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources")
        runner = Runner()
        checks = ["CKV_GITLABCI_1", "CKV_GITLABCI_2"]
        report = runner.run(
            root_folder=valid_dir_path,
            runner_filter=RunnerFilter(framework=['gitlab_ci'], checks=checks)
        )
        self.assertEqual(len(report.failed_checks), 5)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(len(report.passed_checks), 9)
        self.assertEqual(len(report.skipped_checks), 1)
        report.print_console()

    def test_runner_honors_enforcement_rules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources")
        runner = Runner()
        filter = RunnerFilter(framework=['gitlab_ci'], use_enforcement_rules=True)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        filter.enforcement_rule_configs = {CheckType.GITLAB_CI: Severities[BcSeverities.OFF]}
        report = runner.run(
            root_folder=valid_dir_path,
            runner_filter=filter
        )
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 0)
        self.assertEqual(len(report.parsing_errors), 0)

    def test_registry_has_type(self):
        self.assertEqual(registry.report_type, CheckType.GITLAB_CI)

    def test_runner_image_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources")
        runner = Runner()
        report = runner.run(
            root_folder=valid_dir_path,
            runner_filter=RunnerFilter(framework=['gitlab_ci'], checks=['CKV_GITLABCI_3'])
        )
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(len(report.passed_checks), 8)
        self.assertEqual(report.skipped_checks, [])

    def test_runner_image_resources(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources/resource_images")
        runner = Runner()
        report = runner.run(
            root_folder=valid_dir_path,
            runner_filter=RunnerFilter(framework=['gitlab_ci'], checks=['CKV_GITLABCI_3'])
        )
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(len(report.passed_checks), 4)
        self.assertEqual(report.skipped_checks, [])
        self.assertEqual(report.passed_checks[0].resource, 'prebuild')
        self.assertEqual(report.passed_checks[1].resource, 'build.image')
        self.assertEqual(report.passed_checks[2].resource, 'deploy')
        self.assertEqual(report.passed_checks[3].resource, 'deploy')


if __name__ == "__main__":
    unittest.main()



import os
import unittest

from checkov.runner_filter import RunnerFilter
from checkov.secrets.runner import Runner


class TestCustomRegexDetector(unittest.TestCase):

    def test_no_skip(self) -> None:
        """
        Pass regex pattern which is not apply on any file from test_files_dir
        """
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = os.path.join(current_dir, "skip_test/skip_test1/skip_test2")

        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV_SECRET_6:.*3.json$"]))

        summary = report.get_summary()
        self.assertEqual(summary['passed'], 0)
        self.assertEqual(summary['failed'], 10)
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)

    def test_skip_on_one1(self) -> None:
        """
        Pass regex pattern which is apply on one file - first check
        """
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = os.path.join(current_dir, "skip_test/skip_test1/skip_test2")
        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV_SECRET_6:.*1.json$"]))

        summary = report.get_summary()
        self.assertEqual(summary['passed'], 0)
        self.assertEqual(summary['failed'], 5)
        self.assertEqual(summary['skipped'], 5)
        self.assertEqual(summary['parsing_errors'], 0)

    def test_skip_on_one2(self) -> None:
        """
        Pass regex pattern which is apply on one file - second check
        """
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = os.path.join(current_dir, "skip_test/skip_test1/skip_test2")
        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV_SECRET_6:.*2.json$"]))

        summary = report.get_summary()
        self.assertEqual(summary['passed'], 0)
        self.assertEqual(summary['failed'], 5)
        self.assertEqual(summary['skipped'], 5)
        self.assertEqual(summary['parsing_errors'], 0)

    def test_skip_both(self) -> None:
        """
        Pass regex pattern apply all files in root folder
        """
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = os.path.join(current_dir, "skip_test/skip_test1/skip_test2")
        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV_SECRET_6:.*json$"]))

        summary = report.get_summary()
        self.assertEqual(summary['passed'], 0)
        self.assertEqual(summary['failed'], 0)
        self.assertEqual(summary['skipped'], 10)
        self.assertEqual(summary['parsing_errors'], 0)

    def test_skip_directory_pattern(self) -> None:
        """
        Pass regex pattern which is only apply on one directory pattern
        """
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = os.path.join(current_dir, "skip_test/skip_test1")
        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV_SECRET_6:.*skip_test2.*json$"]))

        summary = report.get_summary()
        self.assertEqual(summary['passed'], 0)
        self.assertEqual(summary['failed'], 10)
        self.assertEqual(summary['skipped'], 10)
        self.assertEqual(summary['parsing_errors'], 0)

    def test_skip_file_pattern(self) -> None:
        """
        Pass regex pattern which is only apply on certain file pattern
        """
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = os.path.join(current_dir, "skip_test/skip_test1")
        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV_SECRET_6:.*skip1.json$"]))

        summary = report.get_summary()
        self.assertEqual(summary['passed'], 0)
        self.assertEqual(summary['failed'], 10)
        self.assertEqual(summary['skipped'], 10)
        self.assertEqual(summary['parsing_errors'], 0)

    def test_invalid_regex(self) -> None:
        """
        Pass invalid regex pattern
        """
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = os.path.join(current_dir, "skip_test/skip_test1")
        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV_SECRET_6:[a-z]++$"]))

        summary = report.get_summary()
        self.assertEqual(summary['passed'], 0)
        self.assertEqual(summary['failed'], 20)
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)

    def test_one_good_one_invalid_regex(self) -> None:
        """
        Pass both good & invalid regex pattern
        """
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = os.path.join(current_dir, "skip_test/skip_test1")
        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV_SECRET_6:[a-z]++$", "CKV_SECRET_6:.*skip1.json$"]))

        summary = report.get_summary()
        self.assertEqual(summary['passed'], 0)
        self.assertEqual(summary['failed'], 10)
        self.assertEqual(summary['skipped'], 10)
        self.assertEqual(summary['parsing_errors'], 0)

    def test_two_files_regex_patterns(self) -> None:
        """
        Pass two different regex patterns (file patterns)
        """
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = os.path.join(current_dir, "skip_test/skip_test1")
        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV_SECRET_6:.*skip2.json$", "CKV_SECRET_6:.*skip1.json$"]))

        summary = report.get_summary()
        self.assertEqual(summary['passed'], 0)
        self.assertEqual(summary['failed'], 0)
        self.assertEqual(summary['skipped'], 20)
        self.assertEqual(summary['parsing_errors'], 0)

    def test_two_dir_regex_patterns(self) -> None:
        """
        Pass two different regex patterns (directory related)
        """
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = os.path.join(current_dir, "skip_test")
        report = runner.run(
            root_folder=test_files_dir,
            runner_filter=RunnerFilter(skip_checks=["CKV_SECRET_6:.*skip1.*.json$", "CKV_SECRET_6:.*skip2.*.json$"]))

        summary = report.get_summary()
        self.assertEqual(summary['passed'], 0)
        self.assertEqual(summary['failed'], 0)
        self.assertEqual(summary['skipped'], 30)
        self.assertEqual(summary['parsing_errors'], 0)



# no False Positive - where it's not an actual secret
check_metadata_values = ('bafadssda$#%2', 'bdfsver#$@%')
CHECKOV_METADATA_RESULT = 'checkov_results5243gvr'
check1 = {'blabla': 'blabla1'}
check2 = {'blabla': 'blabla2'}
check1['some_key_1235#$@'] = check2.get('some_value_1235')


access_key = "AKIAIOSFODNN7EXAMPLE"
secret_key = "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"
if __name__ == '__main__':
    print('secrets')


from typing import Any, Dict
import re

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck

_SECURE_RE = re.compile(r"^TLSv1\.(?:2|3)_\d{4}$")


class CloudFrontTLS12(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Verify CloudFront Distribution Viewer Certificate is using TLS v1.2 or higher"
        id = "CKV_AWS_174"
        supported_resources = ("aws_cloudfront_distribution",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        # keep this for reporting
        return "viewer_certificate/[0]/minimum_protocol_version"

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        vc_list = conf.get("viewer_certificate")
        if not isinstance(vc_list, list) or not vc_list or not isinstance(vc_list[0], dict):
            return CheckResult.FAILED

        vc = vc_list[0]

        default_cert = vc.get("cloudfront_default_certificate")
        if isinstance(default_cert, list):
            default_cert = default_cert[0] if default_cert else None
        if isinstance(default_cert, str):
            default_cert = default_cert.lower() == "true"
        if default_cert is True:
            return CheckResult.FAILED

        mpv = vc.get("minimum_protocol_version")
        if isinstance(mpv, list):
            mpv = mpv[0] if mpv else None
        if isinstance(mpv, str) and _SECURE_RE.match(mpv):
            return CheckResult.PASSED

        return CheckResult.FAILED

    def get_evaluated_keys(self):
        return [self.get_inspected_key()]


check = CloudFrontTLS12()



import os
import unittest


from checkov.common.bridgecrew.platform_integration import bc_integration
from checkov.runner_filter import RunnerFilter
from checkov.secrets.runner import Runner
from tests.secrets.utils_for_test import _filter_reports_for_incident_ids


class TestMultilineFinding(unittest.TestCase):

    def test_multiline_finding(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/multiline_finding"
        bc_integration.customer_run_config_response = {"secretsPolicies": [
            {
                "incidentId": "test1",
                "category": "Secrets",
                "severity": "MEDIUM",
                "incidentType": "Violation",
                "title": "test1",
                "guideline": "test",
                "laceworkViolationId": None,
                "prowlerCheckId": None,
                "checkovCheckId": None,
                "resourceTypes":
                    [
                        "aws_instance"
                    ],
                "provider": "AWS",
                "remediationIds":
                    [],
                "customerName": "test1",
                "isCustom": True,
                "code": "definition:\n  cond_type: secrets\n  multiline: true\n  prerun:\n  - (?i)(?:algolia)\n  value:\n  - (?i)(?:algolia)(?:.|[\\n\\r]){0,80}([A-Za-z0-9]{32})\n",
                "descriptiveTitle": None,
                "constructiveTitle": None,
                "pcPolicyId": None,
                "additionalPcPolicyIds": None,
                "pcSeverity": None,
                "sourceIncidentId": None
            }
        ]}
        runner = Runner()
        report = runner.run(root_folder=None,
                            files=[valid_dir_path + "/Dockerfile.mine"],
                            runner_filter=RunnerFilter(framework=['secrets'],
                                                       enable_secret_scan_all_files=True))
        interesting_failed_checks = _filter_reports_for_incident_ids(report.failed_checks, ["test1"])
        self.assertEqual(len(interesting_failed_checks), 1)
        self.assertEqual(len(interesting_failed_checks[0].code_block), 1)
        self.assertEqual(len(interesting_failed_checks[0].code_block[0]), 2)
        self.assertEqual(interesting_failed_checks[0].code_block[0][0], 2)

    def test_multiline_two_secrets_same_check_id(self):
        """
        Regression test: when a file contains 2 secrets matching the same multiline rule
        where the captured group spans multiple real lines, both must be detected with
        correct and distinct line numbers.

        Before fix, find_line_number() cannot find a multiline substring in any
        single line, so it falls back to the prerun match line (line 1) for both secrets.

        Dockerfile.two_multiline_secrets layout:
          line 1:  BEGIN_SECRET   <- prerun matches here
          line 2:  line1_of_secret_aaa...  <- captured group starts here (correct line)
          line 3:  line2_of_secret_aaa...
          line 4:  END_SECRET
          ...
          line 8:  BEGIN_SECRET   <- prerun matches here
          line 9:  line1_of_secret_bbb...  <- captured group starts here (correct line)
          line 10: line2_of_secret_bbb...
          line 11: END_SECRET
        """
        current_dir = os.path.dirname(os.path.realpath(__file__))
        bc_integration.customer_run_config_response = {"secretsPolicies": [
            {
                "incidentId": "test_multiline_two",
                "category": "Secrets",
                "severity": "MEDIUM",
                "incidentType": "Violation",
                "title": "test_multiline_two",
                "guideline": "test",
                "laceworkViolationId": None,
                "prowlerCheckId": None,
                "checkovCheckId": None,
                "resourceTypes": [],
                "provider": "OTHER",
                "remediationIds": [],
                "customerName": "test",
                "isCustom": True,
                "code": "definition:\n  cond_type: secrets\n  multiline: true\n  prerun:\n  - BEGIN_SECRET\n  value:\n  - BEGIN_SECRET\\n((?:.*\\n)+?)END_SECRET\n",
                "descriptiveTitle": None,
                "constructiveTitle": None,
                "pcPolicyId": None,
                "additionalPcPolicyIds": None,
                "pcSeverity": None,
                "sourceIncidentId": None
            }
        ]}
        runner = Runner()
        report = runner.run(root_folder=None,
                            files=[current_dir + "/multiline_finding/Dockerfile.two_secrets"],
                            runner_filter=RunnerFilter(framework=['secrets'],
                                                       enable_secret_scan_all_files=True))
        interesting_failed_checks = _filter_reports_for_incident_ids(report.failed_checks, ["test_multiline_two"])
        # Both secrets must be found
        self.assertEqual(len(interesting_failed_checks), 2)
        lines = sorted(c.file_line_range[0] for c in interesting_failed_checks)
        # The committed fix reports the prerun match line (BEGIN_SECRET) for multiline captured values.
        # First secret: BEGIN_SECRET is on line 1, second: BEGIN_SECRET is on line 8.
        # Before fix, both fall back to line 1 (the first prerun match line).
        self.assertEqual(lines[0], 1)
        self.assertEqual(lines[1], 8)

    def test_single_line_two_secrets_same_check_id(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/single_line_finding"
        bc_integration.customer_run_config_response = {"secretsPolicies": [
            {
                "incidentId": "test_single_two",
                "category": "Secrets",
                "severity": "MEDIUM",
                "incidentType": "Violation",
                "title": "test_single_two",
                "guideline": "test",
                "laceworkViolationId": None,
                "prowlerCheckId": None,
                "checkovCheckId": None,
                "resourceTypes": ["aws_instance"],
                "provider": "AWS",
                "remediationIds": [],
                "customerName": "test",
                "isCustom": True,
                "code": "definition:\n  cond_type: secrets\n  value:\n  - (?i)(?:KEY)\\s*=\\s*'([A-Za-z0-9]{32})'\n",
                "descriptiveTitle": None,
                "constructiveTitle": None,
                "pcPolicyId": None,
                "additionalPcPolicyIds": None,
                "pcSeverity": None,
                "sourceIncidentId": None
            }
        ]}
        runner = Runner()
        report = runner.run(root_folder=None,
                            files=[valid_dir_path + "/secret.txt"],
                            runner_filter=RunnerFilter(framework=['secrets'],
                                                       enable_secret_scan_all_files=True))
        interesting_failed_checks = _filter_reports_for_incident_ids(report.failed_checks, ["test_single_two"])
        # Both secrets must be found
        self.assertEqual(len(interesting_failed_checks), 2)
        lines = sorted(c.file_line_range[0] for c in interesting_failed_checks)
        # First secret is on line 1, second is on line 5
        self.assertEqual(lines[0], 1)
        self.assertEqual(lines[1], 5)



import unittest

from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class TestWildcardEntities(unittest.TestCase):
    def test_contains_unrendered_variable(self):
        self.assertTrue(BaseResourceCheck._is_variable_dependant("var.xyz"))
        self.assertTrue(BaseResourceCheck._is_variable_dependant("local.xyz"))
        self.assertTrue(BaseResourceCheck._is_variable_dependant("module.xyz"))
        self.assertTrue(BaseResourceCheck._is_variable_dependant("${var.xyz}"))
        self.assertTrue(BaseResourceCheck._is_variable_dependant("${local.xyz}"))
        self.assertTrue(BaseResourceCheck._is_variable_dependant("${module.xyz}"))
        self.assertTrue(BaseResourceCheck._is_variable_dependant("aws_ssm_parameter.secret.value"))
        self.assertTrue(BaseResourceCheck._is_variable_dependant("azuread_service_principal_password.gh_actions.value"))
        self.assertTrue(BaseResourceCheck._is_variable_dependant("lookup(var.https_listeners,\"protocol\",\"HTTPS\")"))
        self.assertFalse(BaseResourceCheck._is_variable_dependant("xyz"))
        self.assertFalse(BaseResourceCheck._is_variable_dependant("123"))
        self.assertFalse(BaseResourceCheck._is_variable_dependant(123))
        self.assertFalse(BaseResourceCheck._is_variable_dependant(True))


if __name__ == "__main__":
    unittest.main()



from unittest import mock

import pytest
from checkov.common.images.image_referencer import Image
from checkov.terraform.image_referencer.provider.azure import AzureTerraformProvider
from tests.graph_utils.utils import GRAPH_FRAMEWORKS, \
    set_graph_by_graph_framework, add_vertices_to_graph_by_graph_framework


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_extract_images_from_resources(graph_framework):
    # given
    resource = {
        "file_path_": "/batch.tf",
        "__end_line__": 25,
        "__start_line__": 1,
        "container_configuration": {
            "container_image_names": ["nginx", "python:3.9-alpine"],
            "container_registries": {
                "password": "myPassword",  # checkov:skip=CKV_SECRET_6 test secret
                "registry_server": "myContainerRegistry.azurecr.io",
                "user_name": "myUserName",
            },
            "type": "DockerCompatible",
        },
        "resource_type": "azurerm_batch_pool",
    }
    graph = set_graph_by_graph_framework(graph_framework)
    add_vertices_to_graph_by_graph_framework(graph_framework, resource, graph)

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        azure_provider = AzureTerraformProvider(graph_connector=graph)
        images = azure_provider.extract_images_from_resources()

    # then
    assert images == [
        Image(file_path="/batch.tf", name="nginx", start_line=1, end_line=25, related_resource_id='/batch.tf:None'),
        Image(file_path="/batch.tf", name="python:3.9-alpine", start_line=1, end_line=25, related_resource_id='/batch.tf:None'),
    ]


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_extract_images_from_resources_with_no_image(graph_framework):
    # given
    resource = {
        "file_path_": "/batch.tf",
        "__end_line__": 25,
        "__start_line__": 1,
        "container_configuration": {
            "container_image_names": [],
            "container_registries": {
                "password": "myPassword",
                "registry_server": "myContainerRegistry.azurecr.io",
                "user_name": "myUserName",
            },
            "type": "DockerCompatible",
        },
        "resource_type": "azurerm_batch_pool",
    }
    graph = set_graph_by_graph_framework(graph_framework)
    add_vertices_to_graph_by_graph_framework(graph_framework, resource, graph)

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        azure_provider = AzureTerraformProvider(graph_connector=graph)
        images = azure_provider.extract_images_from_resources()

    # then
    assert not images




from aws_cdk import core
from aws_cdk import aws_redshift as redshift

class MyRedshiftClusterStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Redshift Cluster with logging properties
        redshift.CfnCluster(
            self, 'MyRedshiftCluster',
            cluster_type='single-node',  # Or 'multi-node' based on your configuration
            db_name='mydb',
            master_username='admin',
            master_user_password='password',
            # Other properties as needed for your Redshift cluster
        )

app = core.App()
MyRedshiftClusterStack(app, "MyRedshiftClusterStack")
app.synth()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.StorageSyncPublicAccessDisabled import check
from checkov.common.models.enums import CheckResult


class TestStorageSyncPublicAccessDisabled(unittest.TestCase):

    def test_failure_1(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_storage_sync" "test" {
              name                = "example-storage-sync"
              resource_group_name = azurerm_resource_group.test.name
              location            = azurerm_resource_group.test.location
              tags = {
                foo = "bar"
              }
            }
        """)
        resource_conf = hcl_res['resource'][0]['azurerm_storage_sync']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_2(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_storage_sync" "test" {
              name                = "example-storage-sync"
              resource_group_name = azurerm_resource_group.test.name
              location            = azurerm_resource_group.test.location
              incoming_traffic_policy = "AllowAllTraffic"
              tags = {
                foo = "bar"
              }
            }
        """)
        resource_conf = hcl_res['resource'][0]['azurerm_storage_sync']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_storage_sync" "test" {
              name                = "example-storage-sync"
              resource_group_name = azurerm_resource_group.test.name
              location            = azurerm_resource_group.test.location
              incoming_traffic_policy = "AllowVirtualNetworksOnly"
              tags = {
                foo = "bar"
              }
            }
        """)
        resource_conf = hcl_res['resource'][0]['azurerm_storage_sync']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations

from pathlib import Path

import pytest

from checkov.openapi.runner import Runner
from checkov.runner_filter import RunnerFilter
from tests.common.graph.checks.test_yaml_policies_base import load_yaml_data

BASE_DIR = Path(__file__).parent
CHECK_ID_MAP: "dict[str, str]" = {}  # will be filled via setup()


def test_GlobalSchemeDefineHTTP():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="GlobalSchemeDefineHTTP")


def test_GlobalSecurityScopeUndefined():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="GlobalSecurityScopeUndefined")


def test_Oauth2OperationObjectPasswordFlow():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="Oauth2OperationObjectPasswordFlow")


def test_Oauth2SecurityDefinitionImplicitFlow():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="Oauth2SecurityDefinitionImplicitFlow")


def test_Oauth2SecurityDefinitionPasswordFlow():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="Oauth2SecurityDefinitionPasswordFlow")


def test_Oauth2SecurityPasswordFlow():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="Oauth2SecurityPasswordFlow")


def test_Oauth2SecurityRequirement():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="Oauth2SecurityRequirement")


def test_OperationObjectBasicAuth():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="OperationObjectBasicAuth")


def test_OperationObjectConsumesUndefined():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="OperationObjectConsumesUndefined")


def test_OperationObjectImplicitFlow():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="OperationObjectImplicitFlow")


def test_OperationObjectProducesUndefined():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="OperationObjectProducesUndefined")


def test_OperationObjectSecurityScopeUndefined():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="OperationObjectSecurityScopeUndefined")


def test_PathSchemeDefineHTTP():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="PathSchemeDefineHTTP")


def test_SecurityDefinitionBasicAuth():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="SecurityDefinitionBasicAuth")


def test_SecurityDefinitions():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="SecurityDefinitions")


def test_SecurityRequirement():
    run_check(base_path=BASE_DIR / "resource" / "v2", check="SecurityRequirement")


@pytest.fixture(autouse=True)
def setup():
    global CHECK_ID_MAP
    runner = Runner()
    registry = runner.import_registry()
    CHECK_ID_MAP = {check.__class__.__name__: check.id for entity, check in registry.all_checks()}


def run_check(base_path: Path, check: str) -> None:
    # set path where to find test files
    test_dir_path = base_path / f'example_{check}'

    # setup GitHub configuration runner
    runner = Runner()

    # run actual check
    report = runner.run(runner_filter=RunnerFilter(checks=CHECK_ID_MAP[check]), root_folder=str(test_dir_path))

    # get actual results
    summary = report.get_summary()
    passed_checks = {check.file_path.lstrip("/") for check in report.passed_checks}
    failed_checks = {check.file_path.lstrip("/") for check in report.failed_checks}

    # get expected results
    expected = load_yaml_data(dir_path=test_dir_path, source_file_name="expected.yaml")

    # make sure it is a dict
    assert isinstance(expected, dict)

    expected_to_pass = expected.get("pass") or []
    expected_to_fail = expected.get("fail") or []

    # check, if results are correct
    assert summary["passed"] == len(expected_to_pass)
    assert summary["failed"] == len(expected_to_fail)
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0

    assert passed_checks == set(expected_to_pass)
    assert failed_checks == set(expected_to_fail)



from aws_cdk import core
from aws_cdk import aws_workspaces as workspaces

class WorkSpacesStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create a WorkSpaces directory
        directory = workspaces.CfnWorkspaceDirectory(
            self, "MyWorkspaceDirectory",
            directory_name="my-workspace-directory",
            subnet_ids=["subnet-12345678"],  # Replace with your subnet IDs
            self_service_permissions="ENABLED",
        )

        # Create a WorkSpaces workspace with root volume encryption enabled
        workspace = workspaces.CfnWorkspace(
            self, "MyWorkspace",
            bundle_id="wsb-12345678",  # Replace with your bundle ID
            user_name="my-user",
            root_volume_encryption_enabled=True,
            user_volume_encryption_enabled=False,  # Set to True if you want user volume encryption
            workspace_properties={"directoryId": directory.ref},
        )

app = core.App()
WorkSpacesStack(app, "WorkSpacesStack")
app.synth()



from pathlib import Path
from unittest import mock

import pytest
from pytest_mock import MockerFixture

from checkov.common.bridgecrew.bc_source import get_source_type
from checkov.common.output.report import CheckType
from checkov.runner_filter import RunnerFilter
from checkov.kubernetes.runner import Runner
from tests.common.image_referencer.test_utils import (
    mock_get_empty_license_statuses_async,
    mock_get_license_statuses_async,
    mock_get_image_cached_result_async,
)
from tests.graph_utils.utils import GRAPH_FRAMEWORKS

RESOURCES_PATH = Path(__file__).parent / "resources/k8s"


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_pod_resources(mocker: MockerFixture, graph_framework):
    from checkov.common.bridgecrew.platform_integration import bc_integration

    # given
    file_name = "pod.yaml"
    image_name = "nginx:latest"
    code_lines = "1-14"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    k8s_report = next(report for report in reports if report.check_type == CheckType.KUBERNETES)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(k8s_report.resources) == 1
    assert len(k8s_report.passed_checks) == 69
    assert len(k8s_report.failed_checks) == 20
    assert len(k8s_report.skipped_checks) == 0
    assert len(k8s_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 3
    assert sca_image_report.resources == {
        f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).musl",
        f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).openssl",
        f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).go",
    }
    assert sca_image_report.image_cached_results[0]["dockerImageName"] == "nginx:latest"
    assert (
        "kubernetes/image_referencer/resources/k8s/pod.yaml:Pod.default.webserver"
        in sca_image_report.image_cached_results[0]["relatedResourceId"]
    )
    assert sca_image_report.image_cached_results[0]["packages"] == [
        {"type": "os", "name": "tzdata", "version": "2021a-1+deb11u5", "licenses": []}
    ]

    assert len(sca_image_report.passed_checks) == 1
    assert len(sca_image_report.failed_checks) == 4
    assert len(sca_image_report.image_cached_results) == 1
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_cron_job_resources(mocker: MockerFixture, graph_framework):
    from checkov.common.bridgecrew.platform_integration import bc_integration

    # given
    file_name = "cron_job.yaml"
    image_name = "busybox:1.28"
    code_lines = "1-20"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    k8s_report = next(report for report in reports if report.check_type == CheckType.KUBERNETES)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(k8s_report.resources) == 1
    assert len(k8s_report.passed_checks) == 69
    assert len(k8s_report.failed_checks) == 17
    assert len(k8s_report.skipped_checks) == 0
    assert len(k8s_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 1
    assert sca_image_report.resources == {
        f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 3
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_daemon_set_resources(mocker: MockerFixture, graph_framework):
    from checkov.common.bridgecrew.platform_integration import bc_integration

    # given
    file_name = "daemon_set.yaml"
    image_name = "newrelic/infrastructure"
    code_lines = "11-59"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    k8s_report = next(report for report in reports if report.check_type == CheckType.KUBERNETES)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(k8s_report.resources) == 2
    assert len(k8s_report.passed_checks) == 66
    assert len(k8s_report.failed_checks) == 23
    assert len(k8s_report.skipped_checks) == 0
    assert len(k8s_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 1
    assert sca_image_report.resources == {
        f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 3
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_deployment_resources(mocker: MockerFixture, graph_framework):
    from checkov.common.bridgecrew.platform_integration import bc_integration

    # given
    file_name = "deployment.yaml"
    image_name = "minio/minio:latest"
    code_lines = "13-44"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    k8s_report = next(report for report in reports if report.check_type == CheckType.KUBERNETES)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(k8s_report.resources) == 2
    assert len(k8s_report.passed_checks) == 68
    assert len(k8s_report.failed_checks) == 21
    assert len(k8s_report.skipped_checks) == 0
    assert len(k8s_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 1
    assert sca_image_report.resources == {
        f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 3
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_deployment_config_resources(mocker: MockerFixture, graph_framework):
    from checkov.common.bridgecrew.platform_integration import bc_integration

    # given
    file_name = "deployment_config.yaml"
    image_name = "rhel7/rhel-tools"
    code_lines = "10-26"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    k8s_report = next(report for report in reports if report.check_type == CheckType.KUBERNETES)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(k8s_report.resources) == 2
    assert len(k8s_report.passed_checks) == 65
    assert len(k8s_report.failed_checks) == 14
    assert len(k8s_report.skipped_checks) == 0
    assert len(k8s_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 1
    assert sca_image_report.resources == {
        f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 3
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_job_resources(mocker: MockerFixture, graph_framework):
    from checkov.common.bridgecrew.platform_integration import bc_integration

    # given
    file_name = "job.yaml"
    image_name = "perl:5.34.0"
    code_lines = "1-14"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    k8s_report = next(report for report in reports if report.check_type == CheckType.KUBERNETES)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(k8s_report.resources) == 1
    assert len(k8s_report.passed_checks) == 69
    assert len(k8s_report.failed_checks) == 17
    assert len(k8s_report.skipped_checks) == 0
    assert len(k8s_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 1
    assert sca_image_report.resources == {
        f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 3
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_pod_template_resources(mocker: MockerFixture, graph_framework):
    from checkov.common.bridgecrew.platform_integration import bc_integration

    # given
    file_name = "pod_template.yaml"
    image_name = "alpine"
    code_lines = "1-14"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    k8s_report = next(report for report in reports if report.check_type == CheckType.KUBERNETES)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(k8s_report.resources) == 1
    assert len(k8s_report.passed_checks) == 65
    assert len(k8s_report.failed_checks) == 13
    assert len(k8s_report.skipped_checks) == 0
    assert len(k8s_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 1
    assert sca_image_report.resources == {
        f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 3
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_replica_set_resources(mocker: MockerFixture, graph_framework):
    from checkov.common.bridgecrew.platform_integration import bc_integration

    # given
    file_name = "replica_set.yaml"
    image_name = "gcr.io/google_samples/gb-frontend:v3"
    code_lines = "15-22"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    k8s_report = next(report for report in reports if report.check_type == CheckType.KUBERNETES)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(k8s_report.resources) == 2
    assert len(k8s_report.passed_checks) == 69
    assert len(k8s_report.failed_checks) == 20
    assert len(k8s_report.skipped_checks) == 0
    assert len(k8s_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 1
    assert sca_image_report.resources == {
        f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 3
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_replication_controller_resources(mocker: MockerFixture, graph_framework):
    from checkov.common.bridgecrew.platform_integration import bc_integration

    # given
    file_name = "replication_controller.yaml"
    image_name_1 = "busybox"
    image_name_2 = "quay.io/pires/docker-elasticsearch-kubernetes:5.6.2"
    code_lines_1 = "10-59"
    code_lines_2 = "10-59"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    k8s_report = next(report for report in reports if report.check_type == CheckType.KUBERNETES)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(k8s_report.resources) == 2
    assert len(k8s_report.passed_checks) == 67
    assert len(k8s_report.failed_checks) == 22
    assert len(k8s_report.skipped_checks) == 0
    assert len(k8s_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 2
    assert sca_image_report.resources == {
        f"{file_name} ({image_name_1} lines:{code_lines_1} (sha256:2460522297)).go",
        f"{file_name} ({image_name_2} lines:{code_lines_2} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 6
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_stateful_set_resources(mocker: MockerFixture, graph_framework):
    from checkov.common.bridgecrew.platform_integration import bc_integration

    # given
    file_name = "stateful_set.yaml"
    image_name_1 = "cockroachdb/cockroach-k8s-init:0.2"
    image_name_2 = "cockroachdb/cockroach:v1.1.0"
    code_lines_1 = "14-100"
    code_lines_2 = "14-100"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)
    bc_integration.bc_source = get_source_type("disabled")

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    k8s_report = next(report for report in reports if report.check_type == CheckType.KUBERNETES)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(k8s_report.resources) == 2
    assert len(k8s_report.passed_checks) == 69
    assert len(k8s_report.failed_checks) == 20
    assert len(k8s_report.skipped_checks) == 0
    assert len(k8s_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 2
    assert sca_image_report.resources == {
        f"{file_name} ({image_name_1} lines:{code_lines_1} (sha256:2460522297)).go",
        f"{file_name} ({image_name_2} lines:{code_lines_2} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 6
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0




import unittest

from checkov.arm.registry import arm_resource_registry, arm_parameter_registry


class TestScannerRegistry(unittest.TestCase):

    def test_num_of_scanners(self):
        resource_scanners_counter = 0
        for key in list(arm_resource_registry.checks.keys()):
            resource_scanners_counter += len(arm_resource_registry.checks[key])

        self.assertGreater(resource_scanners_counter, 0)

        parameter_scanners_counter = 0
        for key in list(arm_parameter_registry.checks.keys()):
            parameter_scanners_counter += len(arm_parameter_registry.checks[key])

        self.assertGreater(parameter_scanners_counter, 0)

    def test_non_colliding_check_ids(self):
        check_id_check_class_map = {}
        for (resource_type, checks) in arm_resource_registry.checks.items():
            for check in checks:
                check_id_check_class_map.setdefault(check.id, []).append(check)

        for (resource_type, checks) in arm_parameter_registry.checks.items():
            for check in checks:
                check_id_check_class_map.setdefault(check.id, []).append(check)

        for check_id, check_classes in check_id_check_class_map.items():
            self.assertEqual(len(set(check_classes)), 1)


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.gcp.GoogleFolderMemberDefaultServiceAccount import check
from checkov.common.models.enums import CheckResult


class TestGoogleFolderMemberDefaultServiceAccount(unittest.TestCase):

    def test_failure_binding(self):
        hcl_res = hcl2.loads("""
            resource "google_folder_iam_binding" "folder" {
              folder  = "folders/1234567"
              role    = "roles/editor"
            
              members = [
                "user:jane@example.com",
                "serviceAccount:test-compute@appspot.gserviceaccount.com",
              ]
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_folder_iam_binding']['folder']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_member(self):
        hcl_res = hcl2.loads("""
            resource "google_folder_iam_member" "folder" {
              folder  = "folders/1234567"
              role    = "roles/editor"
              member  = "serviceAccount:test-compute@developer.gserviceaccount.com"
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_folder_iam_member']['folder']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_binding(self):
        hcl_res = hcl2.loads("""
            resource "google_folder_iam_binding" "folder" {
              folder  = "folders/1234567"
              role    = "roles/editor"
            
              members = [
                "user:jane@example.com",
              ]
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_folder_iam_binding']['folder']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_member(self):
        hcl_res = hcl2.loads("""
            resource "google_folder_iam_member" "folder" {
              folder  = "folders/1234567"
              role    = "roles/editor"
              member  = "user:jane@example.com"
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_folder_iam_member']['folder']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)



if __name__ == '__main__':
    unittest.main()
